"use strict";

exports.__esModule = true;
exports.addCustomEnzyme = addCustomEnzyme;
exports.getAcceptedChars = getAcceptedChars;
exports.getClientX = getClientX;
exports.getClientY = getClientY;
exports.getCustomEnzymes = getCustomEnzymes;
exports.getEmptyText = getEmptyText;
exports.getEnzymeAliases = void 0;
exports.getNodeToRefocus = getNodeToRefocus;
exports.getParedDownWarning = getParedDownWarning;
exports.getSelFromWrappedAddon = getSelFromWrappedAddon;
exports.getSelectionMessage = getSelectionMessage;
exports.getStripedPattern = getStripedPattern;
exports.hideAnnByLengthFilter = hideAnnByLengthFilter;
exports.pareDownAnnotations = pareDownAnnotations;
exports.preventDefaultStopPropagation = preventDefaultStopPropagation;
exports.tryToRefocusEditor = tryToRefocusEditor;
var _rangeUtils = require("@teselagen/range-utils");
var _react = _interopRequireDefault(require("react"));
var _proteinUtils = require("./proteinUtils");
var _sequenceUtils = require("@teselagen/sequence-utils");
var _lodash = require("lodash");
var _VeWarning = _interopRequireDefault(require("../helperComponents/VeWarning"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
function getSelectionMessage(_ref) {
  var _ref$caretPosition = _ref.caretPosition,
    caretPosition = _ref$caretPosition === void 0 ? -1 : _ref$caretPosition,
    _ref$selectionLayer = _ref.selectionLayer,
    selectionLayer = _ref$selectionLayer === void 0 ? {
      start: -1,
      end: -1
    } : _ref$selectionLayer,
    customTitle = _ref.customTitle,
    sequenceLength = _ref.sequenceLength,
    sequenceData = _ref.sequenceData,
    showGCContent = _ref.showGCContent,
    GCDecimalDigits = _ref.GCDecimalDigits,
    isProtein = _ref.isProtein;
  var _selectionLayer = selectionLayer;
  var isSelecting = selectionLayer.start > -1;
  if (isSelecting) {
    _selectionLayer = getSelFromWrappedAddon(selectionLayer, sequenceLength);
    var length = (0, _rangeUtils.getRangeLength)(_selectionLayer, sequenceLength);
    var GCContent = function GCContent(numDecimalDigits) {
      if (numDecimalDigits === void 0) {
        numDecimalDigits = 0;
      }
      return (0, _sequenceUtils.calculatePercentGC)((0, _rangeUtils.getSequenceWithinRange)(_selectionLayer, sequenceData.sequence)).toFixed(numDecimalDigits);
    };
    var seqLen = (0, _proteinUtils.divideBy3)(length, isProtein);
    return (customTitle || "Selecting") + " " + seqLen + " " + ((isProtein ? "AA" : "bp") + (seqLen === 1 ? "" : "s")) + " from " + ((0, _proteinUtils.divideBy3)(_selectionLayer.start, isProtein) + 1) + " to " + (0, _proteinUtils.divideBy3)(_selectionLayer.end + 1, isProtein) + (showGCContent && !isProtein ? " (" + GCContent(GCDecimalDigits) + "% GC)" : "");
  } else if (caretPosition > -1) {
    var insertBetween = (0, _sequenceUtils.getInsertBetweenVals)(caretPosition, _selectionLayer, sequenceLength);
    return "Caret Between " + (isProtein ? "AAs " + (0, _proteinUtils.divideBy3)(insertBetween[0], true) + " and " + (0, _proteinUtils.divideBy3)(insertBetween[1] + 2, true) : "Bases " + insertBetween[0] + " and " + insertBetween[1]);
  } else {
    return "No Selection";
  }
}
function preventDefaultStopPropagation(e) {
  if (e) {
    e.stopPropagation();
    e.preventDefault();
  }
}
function getNodeToRefocus(caretEl) {
  var nodeToReFocus;
  if (caretEl && caretEl.closest && caretEl.closest(".veVectorInteractionWrapper")) {
    nodeToReFocus = caretEl.closest(".veVectorInteractionWrapper");
  }
  return nodeToReFocus;
}
function getEmptyText(_ref2) {
  var sequenceData = _ref2.sequenceData,
    caretPosition = _ref2.caretPosition;
  return sequenceData.sequence.length === 0 && caretPosition === -1 ? /*#__PURE__*/_react["default"].createElement("div", {
    className: "veEmptySeqText"
  }, "Insert Sequence Here") : null;
}
function tryToRefocusEditor() {
  var ed = document.querySelector(".veVectorInteractionWrapper");
  ed && ed.focus();
}
function getCustomEnzymes() {
  try {
    var customEnzymes = JSON.parse(window.localStorage.getItem("customEnzymes") || "{}");
    return customEnzymes;
  } catch (error) {
    return {};
  }
}
function addCustomEnzyme(newEnz) {
  var _extends2;
  var customEnzymes = getCustomEnzymes();
  window.localStorage.setItem("customEnzymes", JSON.stringify(_extends({}, customEnzymes, (_extends2 = {}, _extends2[newEnz.name.toLowerCase()] = newEnz, _extends2))));
}
function pareDownAnnotations(annotations, max) {
  var annotationsToPass = annotations;
  var paredDown = false;
  if (Object.keys(annotations).length > max) {
    paredDown = true;
    var sortedAnnotations = (0, _lodash.sortBy)(annotations, function (annotation) {
      return -(0, _rangeUtils.getRangeLength)(annotation);
    });
    annotationsToPass = sortedAnnotations.slice(0, max).reduce(function (obj, item) {
      obj[item.id] = item;
      return obj;
    }, {});
  }
  return [annotationsToPass, paredDown];
}
function getParedDownWarning(_ref3) {
  var nameUpper = _ref3.nameUpper,
    maxToShow = _ref3.maxToShow,
    isAdjustable = _ref3.isAdjustable;
  return /*#__PURE__*/_react["default"].createElement(_VeWarning["default"], {
    key: "ve-warning-max" + nameUpper + "ToDisplay",
    "data-test": "ve-warning-max" + nameUpper + "ToDisplay",
    tooltip: "Warning: More than " + maxToShow + " " + (nameUpper === "Cutsites" ? "Cut Sites" : nameUpper) + ". Only displaying " + maxToShow + " " + (isAdjustable ? "(Configure this under View > Limits)" : "") + " "
  });
}
function getClientX(event) {
  return event.clientX || (0, _lodash.get)(event, "touches[0].clientX");
}
function getClientY(event) {
  return event.clientY || (0, _lodash.get)(event, "touches[0].clientY");
}
function hideAnnByLengthFilter(hideOpts, ann, seqLen) {
  var featLength = (0, _rangeUtils.getRangeLength)(ann, seqLen);
  return hideOpts.enabled && (featLength < hideOpts.min || featLength > hideOpts.max);
}
function getSelFromWrappedAddon(selectionLayer, sequenceLength) {
  var selToUse = _extends({}, selectionLayer);
  if (selectionLayer.isWrappedAddon) {
    var oldEnd = selToUse.end;
    selToUse.end = (0, _rangeUtils.normalizePositionByRangeLength)(selToUse.start - 1, sequenceLength);
    selToUse.start = (0, _rangeUtils.normalizePositionByRangeLength)(oldEnd + 1, sequenceLength);
    delete selToUse.isWrappedAddon;
  }
  return selToUse;
}
function getAcceptedChars(_temp) {
  var _ref4 = _temp === void 0 ? {} : _temp,
    isOligo = _ref4.isOligo,
    isProtein = _ref4.isProtein,
    isRna = _ref4.isRna,
    isMixedRnaAndDna = _ref4.isMixedRnaAndDna;
  return isProtein ? _sequenceUtils.bioData.extended_protein_letters.toLowerCase() : isOligo ? _sequenceUtils.bioData.ambiguous_rna_letters.toLowerCase() + "t" : isRna ? _sequenceUtils.bioData.ambiguous_rna_letters.toLowerCase() : isMixedRnaAndDna ? _sequenceUtils.bioData.ambiguous_rna_letters.toLowerCase() + _sequenceUtils.bioData.ambiguous_dna_letters.toLowerCase() :
  //just plain old dna
  _sequenceUtils.bioData.ambiguous_dna_letters.toLowerCase();
}
function getStripedPattern(_ref5) {
  var color = _ref5.color;
  return /*#__PURE__*/_react["default"].createElement("pattern", {
    id: "diagonalHatch",
    patternUnits: "userSpaceOnUse",
    width: "4",
    height: "4"
  }, /*#__PURE__*/_react["default"].createElement("path", {
    d: "M-1,1 l2,-2 M0,4 l4,-4 M3,5 l2,-2",
    style: {
      stroke: color,
      strokeWidth: 2,
      fill: "blue",
      opacity: 0.4
    }
  }));
}
var getEnzymeAliases = function getEnzymeAliases(enzyme) {
  var lowerName = enzyme.name && enzyme.name.toLowerCase() || "";
  if (typeof enzyme === "string") {
    lowerName = enzyme.toLowerCase();
  }
  return (0, _lodash.filter)((_sequenceUtils.aliasedEnzymesByName[lowerName] || {}).aliases, function (n) {
    return n.toLowerCase() !== lowerName;
  } //filter out current enzyme
  );
};
exports.getEnzymeAliases = getEnzymeAliases;